# frozen_string_literal: true

module Deeds
  module Journals
    class Entry

      attr_reader :input, :node, :error, :line

      def initialize(input:, line: 1)
        @input = input
        @line = line
        parse! # Could be lazy, but that would be too complex
      end

      def dump
        %(#{date} #{minutes}m  \# #{hours.to_f.round(1)} hours)
      end

      def mjd
        date.mjd
      end

      def monday?
        date.monday?
      end

      def sunday?
        date.sunday?
      end

      def work_day?
        !date.saturday? && !date.sunday?
      end

      def date
        node.try!(:date)
      rescue Citrus::ParseError
        nil
      end

      def minutes?
        !minutes.zero?
      end

      def minutes
        node.try!(:minutes)
      rescue Citrus::ParseError
        nil
      end

      def hours
        BigDecimal.new(minutes) / 60
      end

      def activity
        node.try!(:activity)
      rescue Citrus::ParseError
        nil
      end

      private

      def parse!
        @node = Deeds::Grammar.parse input

        # Triggering possible custom Exceptions
        @node.date
        @node.minutes
        @node.activity

      rescue Citrus::ParseError => exception
        @error = exception
      end

    end
  end
end
